<?php
/***********************************************************************************
 * K2DynaMap ---- K2 Dynamic Map Module to plot points specified by the K2 Maps plugin
 *
 * @version		V1.0.0
 * @author		Nick Texidor <nick@texidor.com>
 * @link		http://www.nicktexidor.com
 * @copyright 	Copyright (C) 2008-2010 Nick Texidor. All rights reserved.
 * @license 	http://www.gnu.org/copyleft/gpl.html GNU/GPL 
 * 
 * The K2DynaMap Module is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * K2DynaMap is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with K2DynaMap.  If not, see <http://www.gnu.org/licenses/>.
 *
 *
 * See VERSIONS.TXT for full version history
 *
 **********************************************************************************/

// no direct access
defined('_JEXEC') or die('Restricted access');

require_once (JPATH_SITE.DS.'components'.DS.'com_k2'.DS.'helpers'.DS.'route.php');
require_once (JPATH_SITE.DS.'components'.DS.'com_k2'.DS.'helpers'.DS.'utilities.php');
require_once (JPATH_ADMINISTRATOR.DS.'components'.DS.'com_k2'.DS.'lib'.DS.'k2plugin.php');

class modK2DynaMapHelper
{
	function getList(&$params) {

		$catid = $params->get('catid');

		$user = &JFactory::getUser();
		$aid = $user->get('aid');
		$db = &JFactory::getDBO();

		$jnow = &JFactory::getDate();
		$now = $jnow->toMySQL();
		$nullDate = $db->getNullDate();

		if ($catid != 0) {
			$categories = modK2DynaMapHelper::getCategoryChildren($catid);
			$categories[] = $catid;

			$query = "SELECT * FROM #__k2_items WHERE catid IN (" . implode(',', $categories) . " ) AND published=1 AND ( publish_up = ".$db->Quote($nullDate)." OR publish_up <= ".$db->Quote($now)." ) AND ( publish_down = ".$db->Quote($nullDate)." OR publish_down >= ".$db->Quote($now)." ) AND trash=0 AND access<={$aid} ";
		} else {
			$query = "SELECT * FROM #__k2_items WHERE published=1 AND ( publish_up = ".$db->Quote($nullDate)." OR publish_up <= ".$db->Quote($now)." ) AND ( publish_down = ".$db->Quote($nullDate)." OR publish_down >= ".$db->Quote($now)." ) AND trash=0 AND access<={$aid} ";
		}

		$db->setQuery($query);
		$rows = $db->loadObjectList();

		if ($db->getErrorNum()) {
			echo $db->stderr();
			return false;
		}

		// Create an array of ExtraFields to display if requested 
		if ($params->get('itemextra') == 1) {
			if ($params->get('extrafields') == "") {
				$exfields = array();
			} else {
				$exfields = explode(",", $params->get('extrafields'));

				foreach($exfields as $id=>$value){ 
					$exfields[$id] = trim($value); 
				}
			}
		}


		// Create an array of items to plot 
		$i		= 0;
		$lists	= array();

		foreach ( $rows as $row ) {
			$plugins = new K2Parameter($row->plugins, '', "maps");
			$latitude = trim($plugins->get('latitude'));
			$longitude = trim($plugins->get('longitude'));

			if (($latitude != "") && ($longitude != "")) {
				$lists[$i]->keyref = $latitude . "," . $longitude;

				// Store fields into array for use in Marker Popups //

				// Start with the Title, and the Link.  These are mandatory! //
				$lists[$i]->text = htmlspecialchars( $row->title );
				$lists[$i]->link = JRoute::_(K2HelperRoute::getItemRoute( $row->id . ':' . urlencode($row->alias) ));


				// This is additional free format content for specific record. Can't use Extra Fields 
				// for this because the content would be shown on K2 Item Page //
				$lists[$i]->popuptext = $plugins->get('popuptext');


				// Initialise the optional fields //
				$lists[$i]->introtext 	= "";
				$lists[$i]->itemdate 	= "";
				$lists[$i]->image 		= "";
				$lists[$i]->extrafields = "";


				// Intro Text, truncated if required //
				if ($params->get('itemintro') == 1) {
					$lists[$i]->introtext = $row->introtext;
				
					if ($params->get('introwords')) {
						$lists[$i]->introtext = K2HelperUtilities::wordLimit( $lists[$i]->introtext, $params->get('introwords') );
					}
				}


				// Created Date if required //
				if ($params->get('itemdate') == 1) {
					$lists[$i]->itemdate = htmlspecialchars( $row->created );
				}


				// K2's XSmall image //
				if ($params->get('itemimage') == 1) {
					if (JFile::exists(JPATH_SITE.DS.'media'.DS.'k2'.DS.'items'.DS.'cache'.DS.md5( "Image" . $row->id) . '_XS.jpg')) {
						$lists[$i]->image = JURI::root().'media/k2/items/cache/'.md5("Image" . $row->id).'_XS.jpg';
					}
				}


				// Show ExtarFields if required //
				if ($params->get('itemextra') == 1) {
					$lists[$i]->extrafields = modK2DynaMapHelper::getItemExtraFields($row->extra_fields, $exfields);
				}

				$i++;
			}
		}

		$tmp = array();

		foreach($lists as &$ma) {
			$tmp[] = &$ma->keyref; 
		}

		array_multisort($tmp, $lists); 

/*		foreach($lists as $makey=>&$ma) {
			echo $ma->keyref." - ".$ma->text . " (" . $makey . ") - next ref = " . $lists[$makey + 1]->keyref . "<br/>";
		} */

		return $lists;
	}


	function getCategoryChildren($catid) {

		static $array = array();
		$user = &JFactory::getUser();
		$aid = $user->get('aid');
		$db = &JFactory::getDBO();

		$query = "SELECT * FROM #__k2_categories WHERE parent={$catid} AND published=1 AND trash=0 AND access<={$aid} ORDER BY ordering ";
		$db->setQuery($query);
		$rows = $db->loadObjectList();

		if ($db->getErrorNum()) {
			echo $db->stderr();
			return false;
		}

		foreach ($rows as $row) {
			array_push($array, $row->id);
			if (modK2DynaMapHelper::hasChildren($row->id)) {
				modK2DynaMapHelper::getCategoryChildren($row->id);
			}
		}
    
		return $array;
	}

	function hasChildren($id) {

		$user = &JFactory::getUser();
		$aid = $user->get('aid');
		$db = &JFactory::getDBO();

		$query = "SELECT * FROM #__k2_categories  WHERE parent={$id} AND published=1 AND trash=0 AND access<={$aid} ";
		$db->setQuery($query);
		$rows = $db->loadObjectList();

		if ($db->getErrorNum()) {
			echo $db->stderr();
			return false;
		}

		if (count($rows)) {
			return true;
		} else {
			return false;
		}
	}

	function getItemExtraFields($itemExtraFields, $exfields) {

		jimport('joomla.filesystem.file');
		$db = & JFactory::getDBO ();
		require_once(JPATH_ADMINISTRATOR.DS.'components'.DS.'com_k2'.DS.'lib'.DS.'JSON.php');
		$json=new Services_JSON;
		$jsonObjects=$json->decode($itemExtraFields);
		$imgExtensions = array('jpg','jpeg','gif','png');
		$params = & JComponentHelper::getParams('com_k2');

		if (count($jsonObjects)<1)
		return NULL;

		foreach ($jsonObjects as $object){
			$extraFieldsIDs[]=$object->id;
		}

		$condition=@implode(',',$extraFieldsIDs);

		$query="SELECT * FROM #__k2_extra_fields WHERE published=1 AND id IN ({$condition}) ORDER BY ordering ASC";
		$db->setQuery($query);
		$rawrows = $db->loadObjectList();

		//	Added by Nick 28/6/10 - Filter out unwanted Extra Fields
		for ($i=0; $i<sizeof($rawrows); $i++){

			if ((count($exfields) > 0) && (!in_array($rawrows[$i]->name, $exfields))) {
				continue;
			} else {
				$rows[] = $rawrows[$i];
			}
		}

		for ($i=0; $i<sizeof($rows); $i++){

			$value='';
			$values = array();
			foreach ($jsonObjects as $object){

				if ($rows[$i]->id==$object->id){

					if ( $rows[$i]->type=='textfield'|| $rows[$i]->type=='textarea'){
						$value=$object->value;
					}

					else if($rows[$i]->type=='select'|| $rows[$i]->type=='radio'){
						foreach ($json->decode($rows[$i]->value) as $option){
							if ($option->value==$object->value)
							$value.=$option->name;
						}
					}

					else if ($rows[$i]->type=='multipleSelect'){
						foreach ($json->decode($rows[$i]->value) as $option){
							if (@in_array($option->value,$object->value))
							$values[]=$option->name;
						}
						$value=@implode(', ',$values);
					}

					else if ($rows[$i]->type=='csv'){
						$array = $object->value;
						if(count($array)){
							$value.='<table cellspacing="0" cellpadding="0" class="csvTable">';
							foreach($array as $key=>$row){
								$value.='<tr>';
								foreach($row as $cell){
									$value.=($key>0)?'<td>'.$cell.'</td>':'<th>'.$cell.'</th>';
								}
								$value.='</tr>';
							}
							$value.='</table>';
						}

					}

					else {
						foreach ($json->decode($rows[$i]->value) as $option){

							switch ($object->value[2]){
								case 'same':
								default:
									$attributes='';
									break;

								case 'new':
									$attributes='target="_blank"';
									break;

								case 'popup':
									$attributes='class="classicPopup" rel="{x:'.$params->get('linkPopupWidth').',y:'.$params->get('linkPopupHeight').'}"';
									break;

								case 'lightbox':
									$filename = @basename($object->value[1]);
									$extension = JFile::getExt($filename);
									if (!empty($extension) && in_array($extension,$imgExtensions)) {
										$attributes='class="modal"';
									}
									else {
										$attributes='class="modal" rel="{handler:\'iframe\',size:{x:'.$params->get('linkPopupWidth').',y:'.$params->get('linkPopupHeight').'}}"';
									}
									break;

							}
							$value = $object->value[1] != '' ? '<a href="'.$object->value[1].'" '.$attributes.'>'.$object->value[0].'</a>' : $object->value[0];
						}
					}

				}

			}
			$rows[$i]->value=$value;
		}

		return $rows;
	}
}
